import { parse } from 'node:url';
import encodeUrl from 'encodeurl';
import debug from './debug.js';
export class Redirect {
    #forwardQueryString = false;
    #statusCode = 302;
    #queryString = {};
    #request;
    #response;
    #router;
    #qs;
    constructor(request, response, router, qs) {
        this.#request = request;
        this.#response = response;
        this.#router = router;
        this.#qs = qs;
    }
    #sendResponse(url, query) {
        const stringified = this.#qs.stringify(query);
        url = stringified ? `${url}?${stringified}` : url;
        debug('redirecting to url "%s"', url);
        this.#response.location(encodeUrl(url));
        this.#response.safeStatus(this.#statusCode);
        this.#response.type('text/plain; charset=utf-8');
        this.#response.send(`Redirecting to ${url}`);
    }
    #getReferrerUrl() {
        let url = this.#request.headers['referer'] || this.#request.headers['referrer'] || '/';
        return Array.isArray(url) ? url[0] : url;
    }
    status(statusCode) {
        this.#statusCode = statusCode;
        return this;
    }
    clearQs() {
        this.#forwardQueryString = false;
        this.#queryString = {};
        return this;
    }
    withQs(name, value) {
        if (typeof name === 'undefined') {
            this.#forwardQueryString = true;
            return this;
        }
        if (typeof name === 'string') {
            this.#queryString[name] = value;
            return this;
        }
        Object.assign(this.#queryString, name);
        return this;
    }
    back() {
        let query = {};
        const referrerUrl = this.#getReferrerUrl();
        const url = parse(referrerUrl);
        debug('referrer url "%s"', referrerUrl);
        debug('referrer base url "%s"', url.pathname);
        if (this.#forwardQueryString) {
            query = this.#qs.parse(url.query || '');
        }
        Object.assign(query, this.#queryString);
        this.#sendResponse(url.pathname || '', query);
    }
    toRoute(routeIdentifier, params, options) {
        if (options && options.qs) {
            this.withQs(options.qs);
            options.qs = undefined;
        }
        const url = this.#router.makeUrl(routeIdentifier, params, options);
        return this.toPath(url);
    }
    toPath(url) {
        let query = {};
        if (this.#forwardQueryString) {
            query = this.#qs.parse(parse(this.#request.url).query || '');
        }
        Object.assign(query, this.#queryString);
        this.#sendResponse(url, query);
    }
}
