import { UrlBuilder } from './url_builder.js';
import { RouteFinder } from './route_finder.js';
export class LookupStore {
    #routes = {};
    #encryption;
    #qsParser;
    constructor(encryption, qsParser) {
        this.#encryption = encryption;
        this.#qsParser = qsParser;
    }
    register(route) {
        this.#routes[route.domain] = this.#routes[route.domain] || [];
        this.#routes[route.domain].push(route);
    }
    builder() {
        return this.builderForDomain('root');
    }
    builderForDomain(domain) {
        const routes = this.#routes[domain];
        return new UrlBuilder(this.#encryption, new RouteFinder(routes || []), this.#qsParser);
    }
    find(routeIdentifier, domain) {
        const routes = this.#routes[domain || 'root'] || [];
        return new RouteFinder(routes).find(routeIdentifier);
    }
    findOrFail(routeIdentifier, domain) {
        const routes = this.#routes[domain || 'root'] || [];
        return new RouteFinder(routes).findOrFail(routeIdentifier);
    }
    has(routeIdentifier, domain) {
        const routes = this.#routes[domain || 'root'] || [];
        return new RouteFinder(routes).has(routeIdentifier);
    }
    toJSON() {
        return this.#routes;
    }
}
